#+TITLE: Package-Local Package Nicknames
#+AUTHOR: Nikodemus Siivola and Rudolf Schlatte
#+DATE:      2013-03-06 Wed
#+DESCRIPTION: A CDR specifying package-local package nicknames in Common Lisp
#+KEYWORDS: Lisp, Common Lisp, Packages, Nicknames, Local Nicknames, CDR
#+LANGUAGE:  en
#+OPTIONS:   H:3 num:nil toc:nil \n:nil @:t ::t |:t ^:t -:t f:t *:t <:t
#+OPTIONS:   TeX:t LaTeX:t skip:nil d:nil todo:t pri:nil tags:not-in-toc
#+INFOJS_OPT: view:nil toc:nil ltoc:t mouse:underline buttons:0 path:http://orgmode.org/org-info.js
#+EXPORT_SELECT_TAGS: export
#+EXPORT_EXCLUDE_TAGS: noexport
#+LINK_UP:   
#+LINK_HOME: 
#+XSLT:
#+LaTeX_CLASS: article
#+LaTeX_CLASS_OPTIONS: [a4paper]

* Rationale

Package names and nicknames are global in Common Lisp.  To avoid
collisions, library authors commonly use long package names not unlike
those found in Java, and avoid assigning short nicknames to packages
under their control.

Long package names are not a technical problem per se, since a package's
exported symbols can be imported into other packages and then used
without a package qualifier.  But identically-named symbols can be
exported from multiple packages; the resulting name clashes have to be
resolved, for example via shadowing, and lead to less readable source
code where it is not apparent which of a set of identically-named
functions is called.  Depending on coding style and personal taste, a CL
programmer might also prefer to not import any packages, thereby making
the provenance of any symbols explicit in the program text.

This CDR describes a way to give packages local nicknames.
Package-local package nicknames allow short and easy-to-use names to be
used without fear of name conflict associated with normal, global
package nicknames.

A local nickname is valid only when inside the package for which it has
been specified.  Different packages can use same local nickname for
different global name, or different local nickname for same global
name.  When =*package*= is bound to a package with local nicknames,
calls to =find-package= with any of the local-nicknames will return the
corresponding package, regardless of that package's global name or
nickname(s).  This also affects all implied calls to =find-package=,
including those performed by the reader.

Print-read consistency is preserved in the following way: If a symbol is
not accessible in the current package, and the current package defines a
package-local nickname for the symbol's home package, the symbol is
printed with a package-local nickname as package prefix.  If more than
one package-local nickname names the symbols's home package, it is
unspecified which package prefix will be chosen.

The presence of the keyword =:package-local-nicknames= in =*features*=
denotes support for package-local nicknames.  Specifically, it indicates
that =defpackage= has been extended as documented below, and that
functions listed in [[The Package-Local Nicknames Dictionary][the Package-Local Nicknames Dictionary]] are available in
some implementation-specific extension package.

- defpackage :: name option* \Rightarrow package

     The options to defpackage are extended to include

     =:local-nicknames= (local-nickname actual-package-name)*

     This option adds the specified local nicknames for the
     corresponding actual packages to the package being
     defined or redefined.

Here is an example for the effect of package-local nicknames on
=find-symbol= and =read-from-string=:

#+begin_src lisp
* (defpackage :bar (:intern "X"))
; => #<PACKAGE BAR>
* (defpackage :foo (:intern "X"))
; => #<PACKAGE FOO>
* (defpackage :quux (:use :cl)
    (:local-nicknames (:bar :foo) (:foo :bar)))
; => #<PACKAGE QUUX>
* (find-symbol "X" :foo)
; => FOO::X
; => :INTERNAL
* (read-from-string "BAR::X")
; => BAR::X
; => 6
* (let ((*package* (find-package :quux)))
    (find-symbol "X" :foo))
; => BAR::X
; => :INTERNAL
* (let ((*package* (find-package :quux)))
    (read-from-string "BAR::X"))
; => FOO::X
; => 6
#+end_src

Print-read-consistency is preserved as well.  Observe that the rebinding
of =*package*= is only effective during evaluation, hence the REPL uses
the global package names for printing the symbol =X=.

#+begin_src lisp
* (let ((*package* (find-package :quux)))
    (print (find-symbol "X" :foo)))
; -> FOO::X
; => BAR::X
* (let ((*package* (find-package :quux)))
    (print (read-from-string "BAR::X")))
; -> BAR::X
; => FOO::X
#+end_src

* The Package-Local Nicknames Dictionary
  
An implementation supporting package-local nicknames must provide the
following:
- support for the =:local-nicknames= option to =defpackage=;
- the keyword =:package-local-nicknames= in =*features*=;
- an implementation of the functions =package-local-nicknames=,
  =package-locally-nicknamed-by-list=, =add-package-local-nickname=,
  =remove-package-local-nickname= in an implementation-specific package.
  These symbols should be accessible from the package =CL-USER=.



- package-local-nicknames :: (package-designator) \Rightarrow nicknames

     Returns an alist of =(local-nickname . actual-package)= describing
     the nicknames local to the designated package.  =local-nickname= is
     of type String, =actual-package= is of type Package.


- package-locally-nicknamed-by-list :: (package-designator) \Rightarrow package-list

     Returns a list of packages which have a local nickname for the
     designated package.


- add-package-local-nickname :: (local-nickname actual-package =&optional= package-designator) \Rightarrow package

     Adds =local-nickname= for =actual-package= in the designated
     package, defaulting to the current package. =local-nickname= must
     be a string designator, and =actual-package= must be a package
     designator.

     Returns the designated package.

     Signals an error if =local-nickname= is already a package local
     nickname for a different package, or if =local-nickname= is one of
     ="CL"=, ="COMMON-LISP"=, or ="KEYWORD"=, or if =local-nickname= is
     a global name or nickname for the package to which the nickname
     would be added.

     When in the designated package, calls to =find-package= with
     argument =local-nickname= will return the package the designated
     =actual-package= instead of a package with an identical name or
     nickname. This also affects all implied calls to =find-package=,
     including those performed by the reader.

     When printing a package prefix for a symbol with a package local
     nickname, local nickname is used instead of the real name in order
     to preserve print-read consistency.


- remove-package-local-nickname :: (old-nickname =&optional= package-designator)

     If the designated package had =old-nickname= as a local nickname
     for another package, it is removed. Returns true if the nickname
     existed and was removed, and =nil= otherwise.

* A Note on Package Locks

In Common Lisp implementations which implement package locks, we
recommend that adding or removing a local nickname to a package (not
/for/ a package) should be considered a potential violation of package
locks on that package.

* References and Prior Work

This is not intended to be a comprehensive overview of related ideas
and implementations, merely a summary of the main influences.

** Lisp Machine: local nicknames of packages

See: [[http://common-lisp.net/project/bknr/static/lmman/packd.xml#local%20nicknames%20of%20packages][http://common-lisp.net/project/bknr/static/lmman/packd.xml]]

This is where is all starts, really. Main differences to the interface
specified in this CDR:

- Lisp Machine =defpackage= options are =:relative-names=
  =:relative-names-for-me=, as opposed to =:local-nicknames=.
- Lisp Machine had a special reader syntax for explicitly referring to a
  global package: =#:foo:bar= always referred to the package globally
  named =foo=.

  (XXX: Maybe we should implement this?!)

** More recent implementations

While for one reason or another this facility didn't end up as part of
Common Lisp, it has resurfaced a number of times since then: following
is a small sampling of these:

- [[http://www.cliki.net/cl-package-aliases]]
- [[https://github.com/3b/package-local-nicknames]]
- [[https://github.com/jaeschliman/com.clearly-useful.nick]]
- [[https://github.com/pve1/package-alias]]

The above implementations are either implementation-dependent, or work
via temporary renaming of packages, relying on the user to reset package
nicknames.  None of them address print-read consistency in the presence
of package-local nicknames.  We feel that the deep integration into
printer and reader required for package-local nicknames to work properly
is a good reason for Common Lisp implementors to provide this
functionality.

Finally, to provide more current context, the SBCL mailing list discussion
related to the subject:

[[http://thread.gmane.org/gmane.lisp.steel-bank.devel/16525]]

* Licence

Copyright (c) 2013 Nikodemus Siivola and Rudolf Schlatte

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
